#
# LineUi.rb
#
# Copyright 2005 Tero Hasu
#
# This file defines a primitive line-based user interface for this
# application. The application _should_ be modular enough to allow for
# other kinds of interfaces to be defined also, but this should be
# good enough for now.
#

module ChaosOrder
  class LineUi
    def initialize(form_file)
      @form_file = form_file
      load_form
      @out_file = ext(File.basename(form_file), '.out')
      @out_file.sub!(/\Aform/, 'order')
      @selection = []
    end

    def run
      while true
        $stdout.write "COMMAND: "
        $stdout.flush
        cmd = $stdin.gets.strip
        if cmd =~ /^d\S*\s+(\d+)/i
          increase_quantity($1.to_i, -1)
        elsif cmd =~ /^f\S*\s+(\S.*)/i
          @selection = @model.select_by_title($1)
          print_selection
        elsif cmd =~ /^g\S*\s+(\S.*)/i
          @selection = @model.select_by_page($1)
          print_selection
        elsif cmd =~ /^h/i
          puts %{
commands are
d index (decrease quantity by 1)
f pattern (select by title)
g page (select by page)
h (help)
i index (increase quantity by 1)
p (print order)
quit / exit (quit)
s (print selection)
w (write order to file "#{@out_file}")
}
        elsif cmd =~ /^i\S*\s+(\d+)/i
          increase_quantity($1.to_i, 1)
        elsif cmd =~ /^p/i
          print_order
          puts ''
          puts 'TOTAL QTY: ' + @model.total_qty.to_s
          puts 'TOTAL COST: ' + @model.total_cost
        elsif cmd =~ /^quit$/i or cmd =~ /^exit$/i
          break
        elsif cmd =~ /^s/i
          print_selection
        elsif cmd =~ /^w/i
          save_order
        elsif cmd =~ /^(q|x)/i
          puts 'TYPE quit OR exit TO QUIT'
        else
          puts 'UNKNOWN COMMAND: "%s"' % cmd
        end
      end
    end

    def ext(fn, newext)
      fn.sub(/\.[^.]+$/, newext)
    end

    def increase_quantity index, diff
      if index < 0 or index > (@selection.size - 1)
        puts 'NO ITEM %d IN SELECTION' % index
      else
        @selection[index].qty += diff
      end
      print_selection
    end

    def print_selection
      @selection.each_with_index do |elem, i|
        puts "(%i) %s" % [i, elem.to_s]
      end
    end

    def load_form
      require 'ImportForm'
      data = ChaosOrder::get_form_data_from_file(@form_file)
      require 'ParseForm'
      parser = ChaosOrder::OrderFormParser.new
      parser.parse(data)
      require 'OrderModel'
      @model = ChaosOrder::OrderModel.new(parser.items)
    end

    def save_order
      require 'WriteOrder'
      File.open(@out_file, "w") do |output|
        ChaosOrder::write_order(output, @model)
      end
      puts "Order saved as #{@out_file}"
    end

    def print_order
      require 'WriteOrder'
      ChaosOrder::write_order($stdout, @model)
    end
  end
end

if $0 == __FILE__
  $LOAD_PATH.push(File.dirname(File.expand_path(__FILE__)))
  file = ARGV.shift
  unless file
    # No order form filename given. See if we can find some forms, and
    # if so, choose the most recent one.
    cands = Dir['../forms/form-*']
    file = nil
    best = nil
    for cand in cands
      cand =~ /\/form-(\d\d\d\d_\d\d)\./ or raise
      if (file and ($1 > best)) or (not file)
        file = cand
        best = $1
      end
    end

    # Could not find an order form file, so complain.
    unless file
      puts "USAGE: ruby #{$0} order_form_file"
      exit 1
    end
  end
  ui = ChaosOrder::LineUi.new(file)
  ui.run
end
